@@ -1,5 +1,17 @@
-# Copyright the RTMPy project.
-# See LICENSE.txt for details.
+# Copyright the RTMPy Project
+#
+# RTMPy is free software: you can redistribute it and/or modify it under the
+# terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation, either version 2.1 of the License, or (at your option)
+# any later version.
+#
+# RTMPy is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with RTMPy.  If not, see <http://www.gnu.org/licenses/>.
 
 """
 RTMP implementation.
@@ -111,7 +123,7 @@ class BaseStream(object):
         whenDone = kwargs.get('whenDone', None)
 
         if not whenDone:
-            self.sendMessage(message.Notify(name, *args))
+            self.sendMessage(message.Invoke(name, 0, None, *args))
 
             return
 
@@ -119,29 +131,33 @@ class BaseStream(object):
         invokeId = self.lastInvokeId
 
         d = defer.Deferred()
-        m = message.Invoke(name, invokeId, *args)
+        m = message.Invoke(name, invokeId, None, *args)
         self.activeInvokes[invokeId] = d
 
         self.sendMessage(m, whenDone=whenDone)
 
         return d
 
-    def sendStatus(self, code, *args, **kwargs):
+    def sendStatus(self, code_or_status, command=None, **kwargs):
         """
         Informs the peer of a change of status.
+
+        @param code_or_status: A status message or L{status.Status} instance.
+            If a string is supplied it will be converted to an L{status.Status}.
+        @param command: The command object part of the L{message.Invoke}
+            message. Not quite sure what this achieves right now. Defaults to
+            L{None}.
+        @param kwargs: If a string status message is supplied then any extra
+            kwargs will form part of the generated L{status.Status} message.
         """
-        if isinstance(code, status.Status):
-            kwargs = code.__dict__
+        if isinstance(code_or_status, status.Status):
+            status_obj = code_or_status
         else:
-            kwargs.setdefault('level', 'status')
-            kwargs['code'] = code
+            status_obj = status.status(code_or_status, **kwargs)
 
-        if not args:
-            args = (None,)
+        msg = message.Invoke('onStatus', 0, *[command, status_obj])
 
-        msg = message.Invoke('onStatus', 0, *list(args) + [kwargs])
-
-        self.sendMessage(msg, whenDone=kwargs.get('whenDone', None))
+        self.sendMessage(msg)
 
     def setTimestamp(self, timestamp, relative=True):
         """
@@ -271,14 +287,17 @@ class BaseStream(object):
         """
         self._callExposedMethod(name, *args)
 
+
     def getInvokableTarget(self, name):
         """
-        Returns a callable based on the supplied name, or C{None} if not found.
+        Used to match a callable based on the supplied name when a notify or
+        invoke is encountered. Returns C{None} if not found.
 
         This allows fine grained control over what this stream can expose to the
         peer.
 
         @param name: The name of the function to be mapped to a callable.
+        @return: A callable or C{None}
         """
         func_name = _exposed_funcs.get(name, None)
 
@@ -287,6 +306,7 @@ class BaseStream(object):
 
         return getattr(self, func_name)
 
+
     def sendMessage(self, msg, whenDone=None):
         """
         Sends an RTMP message to the peer. This a low level method and is not
@@ -404,12 +424,14 @@ class RTMPProtocol(protocol.Protocol, BaseStream):
         Called when something fatal has occurred. Logs any errors and closes the
         connection.
         """
-        # weirdly, if log.err is used - trial breaks?!
-        log.msg(failure=reason)
+        log.err(reason)
         self.transport.loseConnection()
 
         return reason
 
+    def closeStream(self):
+        pass
+
     def connectionMade(self):
         """
         Called when this a connection has been made.
@@ -444,6 +466,8 @@ class RTMPProtocol(protocol.Protocol, BaseStream):
                 stream.closeStream()
                 self.deleteStream(streamId)
 
+            self.closeStream()
+
             del_attr('streams')
 
             del_attr('decoder_task')
@@ -567,6 +591,11 @@ class RTMPProtocol(protocol.Protocol, BaseStream):
         @param whenDone: A callback fired when the message has been written to
             the RTMP stream. See L{BaseStream.sendMessage}
         """
+        try:
+            e = self.encoder
+        except AttributeError:
+            return
+
         if stream is None:
             stream = self
 
@@ -577,10 +606,10 @@ class RTMPProtocol(protocol.Protocol, BaseStream):
         # fast enough and the penalty for setting up a new thread is too high.
         msg.encode(buf)
 
-        self.encoder.send(buf.getvalue(), msg.RTMP_TYPE, stream.streamId,
+        e.send(buf.getvalue(), msg.RTMP_TYPE, stream.streamId,
             stream.timestamp, whenDone)
 
-        if not self.encoder_task:
+        if e.active and not self.encoder_task:
             self._startEncoding()
 
     def setFrameSize(self, size):
@@ -592,7 +621,7 @@ class RTMPProtocol(protocol.Protocol, BaseStream):
         """
         self.setFrameSize(4096)
 
-        channel = self.encoder.aquireChannel()
+        channel = self.encoder.acquireChannel()
 
         if not channel:
             # todo: make this better
