@@ -1,5 +1,17 @@
-# Copyright the RTMPy project.
-# See LICENSE.txt for details.
+# Copyright the RTMPy Project
+#
+# RTMPy is free software: you can redistribute it and/or modify it under the
+# terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation, either version 2.1 of the License, or (at your option)
+# any later version.
+#
+# RTMPy is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with RTMPy.  If not, see <http://www.gnu.org/licenses/>.
 
 """
 RTMP codecs. Encoders and decoders for RTMP streams.
@@ -15,7 +27,6 @@ The Encoder/Decoder is not thread safe.
 """
 
 import collections
-import copy
 
 from pyamf.util import BufferedByteStream
 
@@ -116,16 +127,17 @@ class BaseChannel(object):
         else:
             self.header = header.merge(self.header, new)
 
-        # receiving a new message and no timestamp has been supplied means
-        # we use the last known
-        if self._bodyRemaining == -1 and new.timestamp == -1:
-            self.setTimestamp(self._lastDelta)
+        if new.timestamp == -1:
+            # receiving a new message and no timestamp has been supplied means
+            # we use the last known
+            if self.bytes == 0:
+                self.setTimestamp(self._lastDelta, True)
+        else:
+            if not new.continuation:
+                self.setTimestamp(new.timestamp, not new.full)
 
         self._bodyRemaining = self.header.bodyLength - self.bytes
 
-        if new.timestamp > 0:
-            self.setTimestamp(new.timestamp, not new.full)
-
         return old
 
     def _adjustFrameRemaining(self, l):
@@ -189,11 +201,10 @@ class BaseChannel(object):
             self._lastDelta = timestamp
             self.timestamp += timestamp
         else:
-            #if timestamp < self.timestamp:
-            #    raise ValueError('Cannot set a negative timestamp')
+            if self.timestamp == 0:
+                self._lastDelta = timestamp
 
             self.timestamp = timestamp
-            self._lastDelta = 0
 
     def __repr__(self):
         s = []
@@ -333,6 +344,7 @@ class FrameReader(Codec):
     channels to be interleaved together.
     """
 
+    _currentChannel = None
     channel_class = ConsumingChannel
 
     def readHeader(self):
@@ -363,21 +375,32 @@ class FrameReader(Codec):
         C{StopIteration} is raised, otherwise C{IOError}.
         """
         pos = self.stream.tell()
+        channel = self._currentChannel
 
-        try:
-            h = self.readHeader()
+        if channel is None:
+            try:
+                h = self.readHeader()
+            except IOError:
+                self.stream.seek(pos, 0)
+
+                raise StopIteration
+
+            new_pos = self.stream.tell()
+            self.bytes += new_pos - pos
+            pos = new_pos
+
+            channel = self._currentChannel = self.getChannel(h.channelId)
 
-            channel = self.getChannel(h.channelId)
             channel.setHeader(h)
 
+        try:
             bytes = channel.marshallOneFrame()
         except IOError:
             self.stream.seek(pos, 0)
 
-            if self.stream.at_eof():
-                self.stream.consume()
-
             raise StopIteration
+        else:
+            self._currentChannel = None
 
         self.bytes += self.stream.tell() - pos
         complete = channel.complete()
@@ -484,7 +507,12 @@ class Decoder(ChannelDemuxer):
         otherwise C{StopIteration} will be raised if the end of the stream is
         reached.
         """
-        data, meta = ChannelDemuxer.next(self)
+        try:
+            data, meta = ChannelDemuxer.next(self)
+        except StopIteration:
+            self.stream.consume()
+
+            raise
 
         if self.bytes >= self._nextInterval:
             self.dispatcher.bytesInterval(self.bytes)
@@ -526,7 +554,7 @@ class ChannelMuxer(Codec):
 
         self.minChannelId = MIN_CHANNEL_ID
         self.releasedChannels = collections.deque()
-        self.aquiredChannels = []
+        self.acquiredChannels = []
         self.activeChannels = []
         self.internalChannels = {}
         self.channelsInUse = 0
@@ -546,7 +574,7 @@ class ChannelMuxer(Codec):
 
         return property(**locals())
 
-    def aquireChannel(self):
+    def acquireChannel(self):
         """
         Aquires and returns the next available L{Channel} or C{None}.
 
@@ -569,7 +597,7 @@ class ChannelMuxer(Codec):
 
         c = self.getChannel(channelId)
 
-        self.aquiredChannels.append(c)
+        self.acquiredChannels.append(c)
 
         return c
 
@@ -584,7 +612,7 @@ class ChannelMuxer(Codec):
 
         try:
             # FIXME: this is expensive
-            self.aquiredChannels.remove(c)
+            self.acquiredChannels.remove(c)
         except ValueError:
             raise EncodeError('Attempted to release channel %r but that '
                 'channel is not active' % (channelId,))
@@ -654,7 +682,7 @@ class ChannelMuxer(Codec):
             # written right away
             channel = self.getChannel(2)
         else:
-            channel = self.aquireChannel()
+            channel = self.acquireChannel()
 
         if not channel:
             self.pending.append((data, datatype, streamId, timestamp, callback))
@@ -717,6 +745,14 @@ class ChannelMuxer(Codec):
             self.releaseChannel(channel.channelId)
             self.activeChannels.remove(channel)
 
+    @property
+    def active(self):
+        """
+        Whether the encoder is active (i.e. a call to C{next()} will not raise
+        C{StopIteration}.
+        """
+        return self.activeChannels
+
 
 class Encoder(ChannelMuxer):
     """
@@ -797,7 +833,11 @@ class StreamingChannel(object):
 
     def sendData(self, data, timestamp):
         c = self.channel
-        relTimestamp = timestamp - c.timestamp
+
+        if timestamp < c.timestamp:
+            relTimestamp = timestamp
+        else:
+            relTimestamp = timestamp - c.timestamp
 
         h = header.Header(c.channelId, relTimestamp, self.type, len(data), self.streamId)
 
