@@ -1,5 +1,17 @@
-# Copyright the RTMPy project.
-# See LICENSE.txt for details.
+# Copyright the RTMPy Project
+#
+# RTMPy is free software: you can redistribute it and/or modify it under the
+# terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation, either version 2.1 of the License, or (at your option)
+# any later version.
+#
+# RTMPy is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with RTMPy.  If not, see <http://www.gnu.org/licenses/>.
 
 """
 RTMP message implementations.
@@ -300,6 +312,8 @@ class BytesRead(Message):
 
     RTMP_TYPE = BYTES_READ
 
+    FOUR_GB_THRESHOLD = 0xee800000
+
     def __init__(self, bytes=None):
         self.bytes = bytes
 
@@ -317,7 +331,7 @@ class BytesRead(Message):
             raise EncodeError('Bytes read not set')
 
         try:
-            buf.write_ulong(self.bytes)
+            buf.write_ulong(self.bytes % self.FOUR_GB_THRESHOLD)
         except TypeError:
             raise EncodeError('Bytes read wrong type '
                 '(expected int, got %r)' % (type(self.bytes),))
