@@ -1,5 +1,17 @@
-# Copyright the RTMPy Project.
-# See LICENSE.txt for details.
+# Copyright the RTMPy Project
+#
+# RTMPy is free software: you can redistribute it and/or modify it under the
+# terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation, either version 2.1 of the License, or (at your option)
+# any later version.
+#
+# RTMPy is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with RTMPy.  If not, see <http://www.gnu.org/licenses/>.
 
 """
 Server implementation.
@@ -7,7 +19,7 @@ Server implementation.
 
 from zope.interface import Interface, Attribute, implements
 from twisted.internet import protocol, defer
-from twisted.python import failure
+from twisted.python import failure, log
 
 from rtmpy import util, exc, versions
 from rtmpy.protocol.rtmp import message, expose, status
@@ -242,10 +254,12 @@ class NetStream(rtmp.NetStream):
             else:
                 s = status.status('NetStream.Publish.Start',
                     description='%s is now published.' % (name,),
-                    clientid=self.nc.clientId)
+                    clientid=self.client.id)
 
             self.sendStatus(s)
 
+            return result
+
         d.addBoth(send_status)
 
         return d
@@ -261,9 +275,9 @@ class NetStream(rtmp.NetStream):
             d = defer.maybeDeferred(self.nc.unpublishStream, self, self.name)
 
             def send_status(res):
-                self.sendStatus('NetStream.Unpublish.Success',
+                self.sendStatus(status.status('NetStream.Unpublish.Success',
                     description='%s is now unpublished.' % (self.name,),
-                    clientid=self.nc.clientId)
+                    clientid=self.nc.clientId))
 
                 return res
 
@@ -384,9 +398,6 @@ class NetStream(rtmp.NetStream):
     def audioDataReceived(self, data, timestamp):
         self._audioChannel.sendData(data, timestamp)
 
-    def deleteStream(self):
-        """
-        """
 
 
 class ServerProtocol(rtmp.RTMPProtocol):
@@ -408,6 +419,18 @@ class ServerProtocol(rtmp.RTMPProtocol):
         self.application = None
 
     def getInvokableTarget(self, name):
+        """
+        Used to match a callable based on the supplied name when a notify or
+        invoke is encountered. Returns C{None} if not found.
+
+        If no match is found from the superclass, the C{client} and then the
+        C{application} are checked in that order.
+
+        All methods on a client/application is considered B{public} and
+        accessible by the peer.
+
+        @see: L{rtmp.RTMPProtocol.getInvokableTarget}
+        """
         target = rtmp.RTMPProtocol.getInvokableTarget(self, name)
 
         if target:
@@ -417,20 +440,21 @@ class ServerProtocol(rtmp.RTMPProtocol):
         client = getattr(self, 'client', None)
 
         if client:
-            target = getattr(client, name, None)
+            target = util.get_callable_target(client, name)
 
             if target:
                 return target
 
         application = getattr(self, 'application', None)
 
-        if application and hasattr(application.__class__, name):
-            # todo think about protecting some methods?
-            target = getattr(application, name, None)
+        # todo: think about how to protect methods
+        if application:
+            target = util.get_callable_target(application, name)
 
             if target:
                 return target
 
+
     @expose('connect')
     def onConnect(self, args):
         """
@@ -487,7 +511,8 @@ class ServerProtocol(rtmp.RTMPProtocol):
             code = getattr(fail.value, 'code', 'NetConnection.Connect.Failed')
             description = fail.getErrorMessage() or 'Internal Server Error'
 
-            return status.error(code, description)
+            return status.error(code, description,
+                objectEncoding=args.pop('objectEncoding', self.objectEncoding))
 
         def chain_errback(f):
             self._pendingConnection.errback(f)
@@ -596,7 +621,11 @@ class ServerProtocol(rtmp.RTMPProtocol):
         """
         self.application.unpublishStream(streamName, stream)
 
-        return self.application.onUnpublish(self.client, stream)
+        try:
+            self.application.onUnpublish(self.client, stream)
+        except Exception, e:
+            log.err()
+
 
     @expose
     def releaseStream(self, name):
@@ -604,17 +633,32 @@ class ServerProtocol(rtmp.RTMPProtocol):
         Called when the stream is released. Not sure about this one.
         """
 
+
+    def closeStream(self):
+        """
+        Called when the stream is asked to close itself.
+
+        Since this class is considered the B{NetConnection} equivalent, we
+        propagate the event to the attached application (if one exists)
+        """
+        if self.application:
+            self.application.disconnect(self.client)
+
     def playStream(self, name, subscriber, *args):
         """
         """
-        try:
-            publisher = self.application.getStreamByName(name)
-        except:
-            raise exc.StreamNotFound('Unknown stream %r' % (name,))
+        d = defer.Deferred()
 
-        publisher.addSubscriber(subscriber)
+        def whenPublished(publisher):
+            publisher.addSubscriber(subscriber)
 
-        return publisher
+            return publisher
+
+        self.application.whenPublished(name, d.callback)
+
+        d.addCallback(whenPublished)
+
+        return d
 
 
 class StreamPublisher(object):
@@ -637,12 +681,19 @@ class StreamPublisher(object):
 
         self.subscribers = {}
         self.meta = {}
-        self.timestamp = 0
+        self.timestamp = self.baseTimestamp = 0
 
     def _updateTimestamp(self, timestamp):
         """
         """
-        self.timestamp = timestamp
+        if timestamp == 0:
+            self.baseTimestamp = self.timestamp
+
+            return self.timestamp
+
+        self.timestamp = self.baseTimestamp + timestamp
+
+        return self.timestamp
 
     def addSubscriber(self, subscriber):
         """
@@ -659,12 +710,7 @@ class StreamPublisher(object):
         """
         Removes the subscriber from this publisher.
         """
-        print 'removing subscriber'
-
-        try:
-            del self.subscribers[subscriber]
-        except KeyError:
-            pass
+        self.subscribers.pop(subscriber)
 
     # events called by the stream
 
@@ -676,7 +722,7 @@ class StreamPublisher(object):
         @type data: C{str}
         @param timestamp: The timestamp at which this data was received.
         """
-        self._updateTimestamp(timestamp)
+        timestamp = self._updateTimestamp(timestamp)
 
         to_remove = []
 
@@ -686,6 +732,7 @@ class StreamPublisher(object):
             try:
                 subscriber.videoDataReceived(data, relTimestamp)
             except:
+                log.err()
                 to_remove.append(subscriber)
 
         if to_remove:
@@ -700,13 +747,14 @@ class StreamPublisher(object):
         @type data: C{str}
         @param timestamp: The timestamp at which this data was received.
         """
-        self._updateTimestamp(timestamp)
+        timestamp = self._updateTimestamp(timestamp)
         to_remove = []
 
         for subscriber, context in self.subscribers.iteritems():
             try:
                 subscriber.audioDataReceived(data, timestamp - context['timestamp'])
             except:
+                log.err()
                 to_remove.append(subscriber)
 
         if to_remove:
@@ -747,17 +795,22 @@ class Application(object):
     def __init__(self):
         self.clients = {}
         self.streams = {}
+        self._streamingClients = {}
+        self._pendingPublishedCallbacks = {}
+
 
     def startup(self):
         """
         Called when the application is starting up.
         """
 
+
     def shutdown(self):
         """
         Called when the application is closed.
         """
 
+
     def getStreamByName(self, name):
         """
         """
@@ -770,14 +823,35 @@ class Application(object):
         """
         self.clients[client.id] = client
 
+
     def disconnect(self, client):
         """
         Removes the C{client} from this application.
         """
+        publisher = self._streamingClients.pop(client, None)
+
+        if publisher:
+            name = publisher.stream.name
+
+            try:
+                self.unpublishStream(publisher, name)
+            except exc.BadNameError:
+                pass
+            except:
+                log.err()
+
+            self.streams.pop(name, None)
+
         del self.clients[client.id]
 
         client.id = None
 
+        try:
+            self.onDisconnect(client)
+        except Exception, e:
+            log.err()
+
+
     def buildClient(self, protocol, **kwargs):
         """
         Create an instance of a subclass of L{Client}. Override this method to
@@ -792,7 +866,51 @@ class Application(object):
 
         return c
 
-    def publishStream(self, client, stream, name, type_='live'):
+
+    def whenPublished(self, name, cb):
+        """
+        Will call C{cb} when a stream has been published under C{name}
+
+        C{cb} will be called with one argument, the stream object itself.
+        """
+        if not callable(cb):
+            raise TypeError('cb must be callable for whenPublished')
+
+        try:
+            publisher = self.streams[name]
+        except KeyError:
+            cbs = self._pendingPublishedCallbacks.setdefault(name, [])
+
+            cbs.append(cb)
+
+            return
+
+        try:
+            cb(publisher)
+        except:
+            log.err()
+
+
+    def _runCallbacksForPublishedStream(self, name, stream):
+        """
+        Iterates over the list of callables to be executed when a stream named
+        C{name} is successfully published.
+        """
+        try:
+            cbs = self._pendingPublishedCallbacks[name]
+        except KeyError:
+            return
+
+        for cb in cbs:
+            try:
+                cb(stream)
+            except:
+                log.err()
+
+        del self._pendingPublishedCallbacks[name]
+
+
+    def publishStream(self, client, requestor, name, type_='live'):
         """
         The C{stream} is requesting to publish an audio/video stream under the
         name C{name}. Reject the publish request by raising an exception.
@@ -802,16 +920,20 @@ class Application(object):
         @param name: The name of the stream that will be published.
         @param type_: Ignored for now.
         """
-        publisher = self.streams.get(name, None)
+        stream = self.streams.get(name, None)
 
-        if publisher is None:
+        if stream is None:
             # brand new publish
-            publisher = self.streams[name] = StreamPublisher(stream, client)
+            stream = self.streams[name] = StreamPublisher(requestor, client)
+            self._streamingClients[client] = stream
 
-        if client.id != publisher.client.id:
+        if client.id != stream.client.id:
             raise exc.BadNameError('%s is already used' % (name,))
 
-        return publisher
+        self._runCallbacksForPublishedStream(name, stream)
+
+        return stream
+
 
     def unpublishStream(self, name, stream):
         try:
@@ -820,12 +942,16 @@ class Application(object):
             raise exc.BadNameError('Unknown stream %r' % (name,))
 
         if source.client.id != stream.client.id:
-            raise exc.StreamError('Unable to unpublish stream')
+            raise exc.BadNameError('Unable to unpublish stream')
 
-        source.unpublish()
+        try:
+            source.unpublish()
+        except:
+            log.err()
 
         del self.streams[name]
 
+
     def addSubscriber(self, stream, subscriber):
         """
         Adds a subscriber to a stream.
@@ -895,7 +1021,6 @@ class Application(object):
         """
         Called when a client disconnects.
         """
-        self.disconnect(client)
 
 
 class ServerFactory(protocol.ServerFactory):
@@ -953,6 +1078,7 @@ class ServerFactory(protocol.ServerFactory):
             player will use when connecting to this server. An example::
 
             RTMP uri: http://appserver.mydomain.com/webApp; name: webApp.
+
         @param app: The L{IApplication} object that will interact with the
             RTMP clients.
         @return: A deferred signalling the completion of the registration
@@ -996,12 +1122,10 @@ class ServerFactory(protocol.ServerFactory):
         @return: A L{defer.Deferred} when the process is complete. The result
             will be the application instance that was successfully unregistered.
         """
-        try:
-            app = self._pendingApplications.pop(name)
+        app = self._pendingApplications.pop(name, None)
 
+        if app is not None:
             return defer.succeed(app)
-        except KeyError:
-            pass
 
         try:
             app = self.applications[name]
