@@ -1,5 +1,17 @@
-# Copyright (c) The RTMPy Project.
-# See LICENSE.txt for details.
+# Copyright the RTMPy Project
+#
+# RTMPy is free software: you can redistribute it and/or modify it under the
+# terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation, either version 2.1 of the License, or (at your option)
+# any later version.
+#
+# RTMPy is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with RTMPy.  If not, see <http://www.gnu.org/licenses/>.
 
 """
 Tests for L{rtmpy.protocol.rtmp.message}.
@@ -348,6 +360,7 @@ class BytesReadTestCase(BaseTestCase):
         x = message.BytesRead(bytes=20)
         self.assertEquals(x.__dict__, {'bytes': 20})
 
+
     def test_encode(self):
         # test default encode
         x = message.BytesRead()
@@ -367,6 +380,20 @@ class BytesReadTestCase(BaseTestCase):
 
         self.assertEquals(self.buffer.getvalue(), '\x00\x00\x00\x32')
 
+
+    def test_4GB_threshold(self):
+        """
+        Test wrapping when encoding byte values > 4GB.
+
+        @see: #54
+        """
+        x = message.BytesRead(bytes=message.BytesRead.FOUR_GB_THRESHOLD)
+
+        x.encode(self.buffer)
+
+        self.assertEqual(self.buffer.getvalue(), '\x00\x00\x00\x00')
+
+
     def test_decode(self):
         x = message.BytesRead()
 
