@@ -1,5 +1,17 @@
-# Copyright the RTMPy project.
-# See LICENSE.txt for details.
+# Copyright the RTMPy Project
+#
+# RTMPy is free software: you can redistribute it and/or modify it under the
+# terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation, either version 2.1 of the License, or (at your option)
+# any later version.
+#
+# RTMPy is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with RTMPy.  If not, see <http://www.gnu.org/licenses/>.
 
 """
 Tests for L{rtmpy.protocol.rtmp}
@@ -10,7 +22,7 @@ from twisted.internet import error, defer, reactor
 from twisted.test.proto_helpers import StringTransportWithDisconnection
 
 from rtmpy.protocol import rtmp
-from rtmpy.protocol.rtmp import message
+from rtmpy.protocol.rtmp import message, status
 from rtmpy import exc
 
 
@@ -147,6 +159,12 @@ class ConnectionLostTestCase(ProtocolTestCase):
         self.protocol.connectionLost(error.ConnectionDone())
 
 
+
+class TestRuntimeError(RuntimeError):
+    pass
+
+
+
 class CooperateTestCase(ProtocolTestCase):
     """
     Tests for encoding/decoding cooperation
@@ -159,15 +177,19 @@ class CooperateTestCase(ProtocolTestCase):
         self.protocol.handshakeSuccess('')
 
     def test_fail_decode(self):
+        """
+        If something goes wrong whilst decoding, ensure that protocol disconnects
+        """
         def boom(*args):
-            raise RuntimeError
+            raise TestRuntimeError
 
         self.patch(self.protocol.decoder, 'next', boom)
 
         def eb(f):
-            f.trap(RuntimeError)
+            f.trap(TestRuntimeError)
 
             self.assertFalse(self.transport.connected)
+            self.flushLoggedErrors(TestRuntimeError)
 
         d = self.protocol._startDecoding().addErrback(eb)
 
@@ -175,14 +197,15 @@ class CooperateTestCase(ProtocolTestCase):
 
     def test_fail_encode(self):
         def boom(*args):
-            raise RuntimeError
+            raise TestRuntimeError
 
         self.patch(self.protocol.encoder, 'next', boom)
 
         def eb(f):
-            f.trap(RuntimeError)
+            f.trap(TestRuntimeError)
 
             self.assertFalse(self.transport.connected)
+            self.flushLoggedErrors(TestRuntimeError)
 
         d = self.protocol._startEncoding().addErrback(eb)
 
@@ -343,7 +366,7 @@ class BasicResponseTestCase(ProtocolTestCase):
         self.assertEqual(self.messages, [])
 
     def test_sendStatus(self):
-        self.stream.sendStatus('blarg', 'foo', one=1, two='two')
+        self.stream.sendStatus('blarg', 'foo', description='spam', one=1, two='two')
 
         msg, whenDone, stream = self.messages.pop(0)
 
@@ -354,14 +377,16 @@ class BasicResponseTestCase(ProtocolTestCase):
         self.assertIsInstance(msg, message.Invoke)
         self.assertEqual(msg.id, 0)
         self.assertEqual(msg.name, 'onStatus')
-        self.assertEqual(msg.argv, ['foo', {'code': 'blarg', 'level': 'status', 'one': 1, 'two': 'two'}])
+
+        s = status.Status('status', 'blarg', description='spam', one=1, two='two')
+        self.assertEqual(msg.argv, ['foo', s])
 
     def test_send_status_no_args(self):
         """
         If not supplied, the resulting L{message.Invoke} should result in
-        C{argv=[None, {'code': ...}
+        C{argv=[None, <status.Status ...}
         """
-        self.stream.sendStatus('spam')
+        self.stream.sendStatus('spam', description='eggs')
 
         msg, whenDone, stream = self.messages.pop(0)
 
@@ -372,7 +397,30 @@ class BasicResponseTestCase(ProtocolTestCase):
         self.assertIsInstance(msg, message.Invoke)
         self.assertEqual(msg.id, 0)
         self.assertEqual(msg.name, 'onStatus')
-        self.assertEqual(msg.argv, [None, {'code': 'spam', 'level': 'status'}])
+
+        s = status.Status('status', 'spam', description='eggs')
+        self.assertEqual(msg.argv, [None, s])
+
+    def test_send_status_instance(self):
+        """
+        Sending a L{status.Status} instance should allow sending other types of
+        status messages (e.g. level='error')
+        """
+        s = status.Status('error', 'spam', description='eggs')
+
+        self.stream.sendStatus(s)
+
+        msg, whenDone, stream = self.messages.pop(0)
+
+        self.assertEqual(self.messages, [])
+        self.assertIdentical(stream, self.stream)
+        self.assertEqual(whenDone, None)
+
+        self.assertIsInstance(msg, message.Invoke)
+        self.assertEqual(msg.id, 0)
+        self.assertEqual(msg.name, 'onStatus')
+
+        self.assertEqual(msg.argv, [None, s])
 
 
 class InvokableStream(rtmp.NetStream):
